#!/usr/bin/env python
# -*- coding: utf8 -*-

"""Erase the specified annotation types from a FoLiA document, effectively simplifying the document"""

from __future__ import print_function, unicode_literals, division, absolute_import

import argparse
import sys
from foliatools import VERSION as TOOLVERSION
import folia.main as folia


def concat(target, source):
    merges = 0
    for e in source:
        c = e.copy(target.doc)
        target.append(c)
        merges += 1
    return merges



def foliaerase(filename, erasetypes, keepversion=True):
    print("Processing " + filename, file=sys.stderr)
    doc = folia.Document(file=filename, keepversion=keepversion)
    if not keepversion or folia.checkversion(doc.version, "2.0.0") >= 0:
        processor = folia.Processor.create(name="foliaerase",version=TOOLVERSION)
        doc.provenance.append(processor)
    for Class, annotationset in erasetypes:
        count = doc.erase(Class, annotationset)
        print(f"Removed {count} annotations of type {Class.XMLTAG} and set {annotationset}",file=sys.stderr)
    return doc

def parsetype(s):
    if s.lower() in folia.XML2CLASS:
        return folia.XML2CLASS[s]
    try:
        return folia.XML2CLASS[folia.ANNOTATIONTYPE2XML[folia.str2annotationtype(s)]]
    except KeyError:
        raise ValueError("No such annotation type: " + s)

def parsetypes(s):
    pairs = s.split(";")
    for pair in pairs:
        pair = pair.strip()
        if ',' in pair:
            t, s = pair.split(',')
            yield (parsetype(t.strip()), s)
        else:
            yield (parsetype(pair.strip()), False)

def main():
    parser = argparse.ArgumentParser(description=__doc__, formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('-v','--version',help="Show version information", action='version', version="FoLiA-tools v" + TOOLVERSION + ", using FoLiA v" + folia.FOLIAVERSION + " with library FoLiApy v" + folia.LIBVERSION, default=False)
    parser.add_argument('-n','--dryrun', help="Do not write files", action='store_true')
    parser.add_argument('-O','--output', help="Output to standard output", action='store_true')
    parser.add_argument('-u','--upgrade', help="Automatically upgrade FoLiA version to the latest version, warning: make sure you validate your document afterwards as this does not always work for large upgrades! Preferably use the foliaupgrade tool instead", action='store_true')
    parser.add_argument('-t','--types', type=str, help="Annotation types to erase. This is a semicolon separated list of annotation types (you can use the primary FoLiA XML tags), set names are separated from the annotation type by a comma.", required=True )
    parser.add_argument('files', nargs='*', help='Files to process')
    args = parser.parse_args()

    if len(args.files) < 1:
        print("No files specified! Do --help for usage info!", file=sys.stderr)

    for file in args.files:
        doc = foliaerase(file, parsetypes(args.types), keepversion=not args.upgrade)
        if not args.dryrun:
            doc.save()
        if args.output:
            print(doc.xmlstring())

if __name__ == "__main__":
    main()



#!/usr/bin/env python
#-*- coding:utf-8 -*-

"""
This conversion script reads a FoLiA XML document and outputs the
document's text as plain text, *without* any annotations.
Use folia2annotatedtxt if you want limited support for inline annotations.
"""

from __future__ import print_function, unicode_literals, division, absolute_import

import getopt
import io
import sys
import os
import glob
import folia.main as folia

def usage():
    print("folia2txt",file=sys.stderr)
    print("  by Maarten van Gompel (proycon)",file=sys.stderr)
    print("  Centre for Language and Speech Technology, Radboud University Nijmegen",file=sys.stderr)
    print("  2012-2019 - Licensed under GPLv3",file=sys.stderr)
    print("",file=sys.stderr)
    print(__doc__,file=sys.stderr)
    print("",file=sys.stderr)
    print("Usage: folia2txt [options] file-or-dir1 file-or-dir2 ..etc..",file=sys.stderr)
    print("",file=sys.stderr)
    print("Parameters for output:",file=sys.stderr)
    print("  -t                           Retain tokenisation, do not detokenise",file=sys.stderr)
    print("                               (By default output will be detokenised if",file=sys.stderr)
    print("                               such information is explicitly available in the",file=sys.stderr)
    print("                               FoLiA document)",file=sys.stderr)
    print("  -c                           Text class to output (defaults to: current)",file=sys.stderr)
    print("  -w                           One word per line",file=sys.stderr)
    print("  -s                           One sentence per line",file=sys.stderr)
    print("  -p                           One paragraph per line",file=sys.stderr)
    print("  -o [filename]                Output to a single file (instead of default stdout)",file=sys.stderr)
    print("  -e [encoding]                Output encoding (default: utf-8)",file=sys.stderr)
    print("Parameters for processing directories:",file=sys.stderr)
    print("  -r                           Process recursively",file=sys.stderr)
    print("  -E [extension]               Set extension (default: xml)",file=sys.stderr)
    print("  -O                           Output each file to similarly named .txt file",file=sys.stderr)
    print("  -P                           Like -O, but outputs to current working directory",file=sys.stderr)
    print("  -q                           Ignore errors",file=sys.stderr)

def out(s, outputfile):
    if sys.version < '3':
        if outputfile:
            outputfile.write(s + "\n")
        else:
            print(s.encode(settings.encoding))
    else:
        if outputfile:
            print(s,file=outputfile)
        else:
            print(s)


def process(filename, outputfile = None):
    print("Converting " + filename,file=sys.stderr)
    try:
        doc = folia.Document(file=filename)

        if settings.autooutput:
            if filename[-len(settings.extension) - 1:].lower() == '.' +settings.extension:
                outfilename = filename[:-len(settings.extension) - 1] + '.txt'
            else:
                outfilename += '.txt'
            if settings.autooutput_cwd:
                outfilename = os.path.basename(outfilename)

            print(" Saving as " + outfilename,file=sys.stderr)
            outputfile = io.open(outfilename,'w',encoding=settings.encoding)

        if settings.wordperline:
            for word in doc.words():
                out(word.text(settings.textclass, settings.retaintokenisation), outputfile)
        elif settings.sentenceperline:
            for sentence in doc.sentences():
                out(sentence.text(settings.textclass, settings.retaintokenisation) , outputfile)
        elif settings.paragraphperline:
            for paragraph in doc.paragraphs():
                out(paragraph.text(settings.textclass, settings.retaintokenisation) , outputfile)
        else:
            out(doc.text(settings.textclass, settings.retaintokenisation) , outputfile)

        if settings.autooutput:
            outputfile.close()
        elif outputfile:
            outputfile.flush()
    except Exception as e:
        if settings.ignoreerrors:
            print("ERROR: An exception was raised whilst processing " + filename + ":", e, file=sys.stderr)
        else:
            raise




def processdir(d, outputfile = None):
    print("Searching in  " + d, file=sys.stderr)
    for f in glob.glob(os.path.join(d, '*')):
        if f[-len(settings.extension) - 1:] == '.' + settings.extension:
            process(f, outputfile)
        elif settings.recurse and os.path.isdir(f):
            processdir(f, outputfile)


class settings:
    wordperline = False
    sentenceperline = False
    paragraphperline = False
    retaintokenisation = False
    autooutput = False
    autooutput_cwd = False
    extension = 'xml'
    recurse = False
    ignoreerrors = False
    encoding = 'utf-8'
    textclass = "current"


def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "o:OPE:htspwrqc:", ["help"])
    except getopt.GetoptError as err:
        print(str(err), file=sys.stderr)
        usage()
        sys.exit(2)


    outputfile = None


    for o, a in opts:
        if o == '-h' or o == '--help':
            usage()
            sys.exit(0)
        elif o == '-t':
            settings.retaintokenisation = True
        elif o == '-e':
            settings.encoding = a
        elif o == '-E':
            settings.extension = a
        elif o == '-o':
            outputfile = a
        elif o == '-O':
            settings.autooutput = True
        elif o == '-P':
            settings.autooutput = True
            settings.autooutput_cwd = True
        elif o == '-c':
            settings.textclass = a
        elif o == '-s':
            settings.sentenceperline = True
        elif o == '-p':
            settings.paragraphperline = True
        elif o == '-w':
            settings.wordperline = True
        elif o == '-r':
            settings.recurse = True
        elif o == '-q':
            settings.ignoreerrors = True
        else:
            raise Exception("No such option: " + o)


    if outputfile: outputfile = io.open(outputfile,'w',encoding=settings.encoding)

    if args:
        for x in args:
            if os.path.isdir(x):
                processdir(x,outputfile)
            elif os.path.isfile(x):
                process(x, outputfile)
            else:
                print("ERROR: File or directory not found: " + x, file=sys.stderr)
                sys.exit(3)
    else:
        print("ERROR: Nothing to do, specify one or more files or directories", file=sys.stderr)

    if outputfile: outputfile.close()

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
#-*- coding:utf-8 -*-

#---------------------------------------------------------------
# TEI to FoLiA Converter
#   by Maarten van Gompel
#   Centre for Language Studies
#   Radboud University Nijmegen
#   proycon AT anaproy DOT nl
#
#   Licensed under GPLv3
#----------------------------------------------------------------

"""Converts plain text to FoLiA XML."""

import sys
import argparse
import os.path
import lxml.etree
import traceback
import time
from socket import getfqdn
from datetime import datetime
from urllib.parse import  urlparse
from urllib.request import urlretrieve
import folia.main as folia
from foliatools import VERSION as TOOLVERSION


def convert(filename, **kwargs):
    if not os.path.exists(filename):
        raise Exception("File not found: " + filename)

    if 'id' in kwargs and kwargs['id']:
        docid = folia.makencname(kwargs['id'])
    else:
        docid = folia.makencname(os.path.basename(filename).split('.')[0])
    doc = folia.Document(id=docid, processor=folia.Processor.create(name="txt2folia", version=TOOLVERSION) )
    body = doc.append(folia.Text(doc, id=docid + '.text'))
    buffer = []
    with open(filename,'r',encoding=kwargs['encoding']) as f:
        for line in f:
            if kwargs.get('sentenceperline'):
                if line.strip():
                    body.append(folia.Sentence, line.strip())
            elif kwargs.get('paragraphperline'):
                if line.strip():
                    body.append(folia.Paragraph, line.strip())
            elif kwargs.get('nostructure'):
                if not kwargs.get('nolinebreaks'):
                    if line.strip():
                        buffer.append( line.strip() )
                    if buffer and isinstance(buffer[-1],str) and buffer[-1][-1] != '-':
                        buffer.append(folia.Linebreak(doc))
                else:
                    if buffer and isinstance(buffer[-1],str) and buffer[-1][-1] not in (' ','-'):
                        buffer.append( " " + line.strip() )
                    else:
                        buffer.append( line.strip() )
            else:
                if not line.strip():
                    #empty line, add buffer
                    body.append(folia.Paragraph, folia.TextContent(doc,*buffer))
                    buffer = []
                else:
                    buffer.append(line.strip())
                    if not kwargs.get('nolinebreaks'):
                        buffer.append(folia.Linebreak(doc))

    if buffer:
        if kwargs.get('nostructure'):
            body.data = [folia.TextContent(doc, *buffer )]
        else:
            body.append(folia.Paragraph, folia.TextContent(doc,*buffer))

    return doc

def main():
    parser = argparse.ArgumentParser(description=__doc__, formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('-o','--outputdir',type=str, help="Output directory, set to - for stdout", action="store",default=".",required=False)
    parser.add_argument('--id',type=str,help="Document ID", action='store')
    parser.add_argument('--encoding',type=str,help="Encoding", action='store',default='utf-8')
    parser.add_argument('--sentenceperline',help="Assume one sentence per line", action='store_true')
    parser.add_argument('--paragraphperline',help="Assume one paragraph per line", action='store_true')
    parser.add_argument('--paragraphs',help="Assumes paragraphs seperated by an empty line (this is the default)", action='store_true')
    parser.add_argument('--nostructure',help="Do not extract any structure, just wrap the entire text in a minimal FoLiA structure", action='store_true')
    parser.add_argument('--nolinebreaks',help="Do not explicitly encode linebreaks", action='store_true')
    parser.add_argument('files', nargs='+', help='Text files to process')
    args = parser.parse_args()

    for filename in args.files:
        print("Converting", filename,file=sys.stderr)
        doc = convert(filename, **args.__dict__)
        if doc is False:
            print("Unable to convert ", filename,file=sys.stderr)
            sys.exit(1) #an error occured
        if args.outputdir == "-":
            print(doc.xmlstring())
        else:
            filename = os.path.basename(filename)
            if filename[-4:] == '.xml':
                filename = filename[:-4] + '.folia.xml'
            else:
                filename += '.folia.xml'
            print("   Writing", filename,file=sys.stderr)
            doc.save(os.path.join(args.outputdir, filename))
    sys.exit(0)

if __name__ == '__main__':
    main()




#!/usr/bin/env python3

"""Automatically upgrade older FoLiA documents to the latest version. Also has the ability to apply certain fixes."""

import sys
import os.path
import argparse
import glob
import shutil
import folia.main as folia
from socket import getfqdn
from foliatools import VERSION as TOOLVERSION
from foliatools.foliavalidator import validate


def main():
    parser = argparse.ArgumentParser(description=__doc__, formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('-v','-V','--version',help="Show version information", action='version', version="FoLiA-tools v" + TOOLVERSION + ", using FoLiA v" + folia.FOLIAVERSION + " with library FoLiApy v" + folia.LIBVERSION, default=False)
    parser.add_argument('-n', '--dryrun',help="Dry run, do not write files", action='store_true', default=False)
    parser.add_argument('-E','--extension', type=str,help="Extension", action='store',default="xml",required=False)
    parser.add_argument('--fixunassignedprocessor',help="Fixes invalid FoLiA that does not explicitly assign a processor to an annotation when multiple processors are possible (and there is therefore no default). The last processor will be used in this case.", action='store_true', default=False)
    parser.add_argument('files', nargs='+', help='Input files')
    args = parser.parse_args()

    success = process(*args.files, **args.__dict__)
    sys.exit(0 if success else 1)

def convert_set(annotationtype, annotationset):
    if annotationset == "undefined":
        if annotationtype == folia.AnnotationType.TEXT:
            return folia.DEFAULT_TEXT_SET
        elif annotationtype == folia.AnnotationType.PHON:
            return folia.DEFAULT_PHON_SET
        else:
            return None
    else:
        return annotationset

def convert_undefined_sets(doc, elements = None):
    if elements is None: elements = doc.elements()
    exempt = set()
    for element in elements:
        if element.set == "undefined" and element.cls:
            exempt.add(element.ANNOTATIONTYPE)
    for element in elements:
        if element.set == "undefined" and element.ANNOTATIONTYPE not in exempt:
            element.set = None
    doc.annotations = [ (annotationtype, convert_set(annotationtype, annotationset) ) for annotationtype, annotationset in doc.annotations ]


def annotators2processors(doc, mainprocessor, elements=None):
    """Convert FoLiA v1 style annotators to v2 style processors (limited)"""
    if elements is None: elements = doc.elements()
    for element in doc.elements():
        if element.annotator is not None:
            element.annotator2processor(element.annotator, element.annotatortype, parentprocessor=mainprocessor) #the library does the bulk of the work for us
        elif element.annotatortype is not None:
            element.annotator2processor("unspecified", element.annotatortype, parentprocessor=mainprocessor) #the library does the bulk of the work for us
    doc.annotationdefaults = {} #not needed anymore

def upgrade(doc, upgradeprocessor):
    assert doc.autodeclare

    if not doc.provenance or (len(doc.provenance) == 1 and doc.provenance.processors[0] == upgradeprocessor):
        #add a datasource processor as the first one
        doc.provenance.insert(0, folia.Processor(doc.id + ".pre-upgrade", folia_version=doc.version, type=folia.ProcessorType.DATASOURCE, host=getfqdn(), src="file://" + doc.filename, format="text/folia+xml"))

    #bump the version number
    doc.version = folia.FOLIAVERSION
    elements = doc.elements()
    #convert old style annotators to new style processors
    annotators2processors(doc, upgradeprocessor, elements)
    #convert undefined sets
    convert_undefined_sets(doc, elements)


def process(*files, **kwargs):
    success = False
    for file in files:
        if os.path.isdir(file):
            files = list(glob.glob(os.path.join(file, "*." + kwargs['extension'] )) )
            r = process(files, **kwargs)
            if files and r:
                success = True
        elif os.path.isfile(file):
            mainprocessor = folia.Processor.create(name="foliaupgrade", version=TOOLVERSION, src="https://github.com/proycon/foliatools")
            doc = validate(file,schema=None, stricttextvalidation=True,autodeclare=True,output=False, warn=False,processor=mainprocessor,traceback=True,**kwargs)
            if doc is not False:
                print("Upgrading " + doc.filename,file=sys.stderr)
                upgrade(doc, mainprocessor)
                doc.save(doc.filename + ".upgraded")
                if not validate(file + ".upgraded",schema=None,stricttextvalidation=True,autodeclare=False,traceback=True,**kwargs):
                    print("Upgrade failed",file=sys.stderr)
                    success = False
                else:
                    success = True
                    print("Upgrade OK",file=sys.stderr)
                    if kwargs.get('dryrun'):
                        os.unlink(doc.filename+".upgraded")
                        print(doc.xmlstring())
                    else:
                        shutil.move(doc.filename + ".upgraded", file)
            else:
                success  = False
    return success

if __name__ == "__main__":
    main()
